

# class Violation(object):
# 	def  __init__(self, fileName, configInfo):
# 		self.fileName = fileName
# 		self.configInfo = configInfo
# 		self.lineNum = 0
# 		self.columnNum = 0

class Violation(object):
	"""docstring for ClassName"""
	def __init__(self, path, configinfo={}):
		super(Violation, self).__init__()
		self.path = path
		self.configinfo = configinfo
		self.startline = 0
		self.startcolumn = 0
		self.message = ""
		self.__rulename = ""
		self.__rulecategory = ""
		self.__democode = ""
		self.__noncomliant = ""
		self.__priority = "3"
		self.currCode = ""

	

	def get_rulename(self):
		if self.configinfo.has_key('RuleName'):
			return self.configinfo['RuleName']
		else:
			return self.__rulename

	def set_rulename(self, rulename):
		self.__rulename = rulename

	def get_rulecategory(self):

		if self.configinfo.has_key('RuleCategory'):
			return self.configinfo['RuleCategory']
		else:
			return self.__rulecategory

	def set_rulecategory(self, rulecategory):
		self.__rulecategory = rulecategory


	def get_priority(self):

		if self.configinfo.has_key('Priority'):
			return self.configinfo['Priority']
		else:
			return self.__priority

	def set_priority(self, priority):
		self.__priority = priority

	def get_democode(self):

		if self.configinfo.has_key('DemoCode'):
			return self.configinfo['DemoCode']
		else:
			return self.__democode

	def set_democode(self, democode):
		self.__democode = democode

	def get_noncompliant_code(self):
		if self.configinfo.has_key('Noncomliant'):
			return self.configinfo['Noncomliant']
		else:
			return self.__noncomliant

	def set_noncompliant_code(self, noncomliant):
		self.__noncomliant = noncomliant





	
		




#
# Copyright (c) 2012-2017 The ANTLR Project. All rights reserved.
# Use of this file is governed by the BSD 3-clause license that
# can be found in the LICENSE.txt file in the project root.
#

#
# Provides an implementation of {@link TokenSource} as a wrapper around a list
# of {@link Token} objects.
#
# <p>If the final token in the list is an {@link Token#EOF} token, it will be used
# as the EOF token for every call to {@link #nextToken} after the end of the
# list is reached. Otherwise, an EOF token will be created.</p>
#
from QBCodeSpecs.antlr4.CommonTokenFactory import CommonTokenFactory
from QBCodeSpecs.antlr4.Lexer import TokenSource
from QBCodeSpecs.antlr4.Token import Token


class ListTokenSource(TokenSource):

    # Constructs a new {@link ListTokenSource} instance from the specified
    # collection of {@link Token} objects and source name.
    #
    # @param tokens The collection of {@link Token} objects to provide as a
    # {@link TokenSource}.
    # @param sourceName The name of the {@link TokenSource}. If this value is
    # {@code null}, {@link #getSourceName} will attempt to infer the name from
    # the next {@link Token} (or the previous token if the end of the input has
    # been reached).
    #
    # @exception NullPointerException if {@code tokens} is {@code null}
    #
    def __init__(self, tokens, sourceName=None):
        if tokens is None:
            raise ReferenceError("tokens cannot be null")
        self.tokens = tokens
        self.sourceName = sourceName
        # The index into {@link #tokens} of token to return by the next call to
        # {@link #nextToken}. The end of the input is indicated by this value
        # being greater than or equal to the number of items in {@link #tokens}.
        self.pos = 0
        # This field caches the EOF token for the token source.
        self.eofToken = None
        # This is the backing field for {@link #getTokenFactory} and
        self._factory = CommonTokenFactory.DEFAULT


    #
    # {@inheritDoc}
    #
    @property
    def column(self):
        if self.pos < len(self.tokens):
            return self.tokens[self.pos].column
        elif self.eofToken is not None:
            return self.eofToken.column
        elif len(self.tokens) > 0:
            # have to calculate the result from the line/column of the previous
            # token, along with the text of the token.
            lastToken = self.tokens[len(self.tokens) - 1]
            tokenText = lastToken.text
            if tokenText is not None:
                lastNewLine = tokenText.rfind('\n')
                if lastNewLine >= 0:
                    return len(tokenText) - lastNewLine - 1
            return lastToken.column + lastToken.stop - lastToken.start + 1

        # only reach this if tokens is empty, meaning EOF occurs at the first
        # position in the input
        return 0

    #
    # {@inheritDoc}
    #
    def nextToken(self):
        if self.pos >= len(self.tokens):
            if self.eofToken is None:
                start = -1
                if len(self.tokens) > 0:
                    previousStop = self.tokens[len(self.tokens) - 1].stop
                    if previousStop != -1:
                        start = previousStop + 1
                stop = max(-1, start - 1)
                self.eofToken = self._factory.create((self, self.getInputStream()),
                            Token.EOF, "EOF", Token.DEFAULT_CHANNEL, start, stop, self.line, self.column)
            return self.eofToken
        t = self.tokens[self.pos]
        if self.pos == len(self.tokens) - 1 and t.type == Token.EOF:
            self.eofToken = t
        self.pos += 1
        return t

    #
    # {@inheritDoc}
    #
    @property
    def line(self):
        if self.pos < len(self.tokens):
            return self.tokens[self.pos].line
        elif self.eofToken is not None:
            return self.eofToken.line
        elif len(self.tokens) > 0:
            # have to calculate the result from the line/column of the previous
            # token, along with the text of the token.
            lastToken = self.tokens[len(self.tokens) - 1]
            line = lastToken.line
            tokenText = lastToken.text
            if tokenText is not None:
                line += tokenText.count('\n')

            # if no text is available, assume the token did not contain any newline characters.
            return line

        # only reach this if tokens is empty, meaning EOF occurs at the first
        # position in the input
        return 1

    #
    # {@inheritDoc}
    #
    def getInputStream(self):
        if self.pos < len(self.tokens):
            return self.tokens[self.pos].getInputStream()
        elif self.eofToken is not None:
            return self.eofToken.getInputStream()
        elif len(self.tokens) > 0:
            return self.tokens[len(self.tokens) - 1].getInputStream()
        else:
            # no input stream information is available
            return None

    #
    # {@inheritDoc}
    #
    def getSourceName(self):
        if self.sourceName is not None:
            return self.sourceName
        inputStream = self.getInputStream()
        if inputStream is not None:
            return inputStream.getSourceName()
        else:
            return "List"



# Generated from ObjectiveCPreprocessorParser.g4 by ANTLR 4.7.1
from QBCodeSpecs.antlr4 import *

# This class defines a complete listener for a parse tree produced by ObjectiveCPreprocessorParser.
class ObjectiveCPreprocessorParserListener(ParseTreeListener):

    # Enter a parse tree produced by ObjectiveCPreprocessorParser#preprocessorImport.
    def enterPreprocessorImport(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCPreprocessorParser#preprocessorImport.
    def exitPreprocessorImport(self, ctx):
        print ctx
        pass


    # Enter a parse tree produced by ObjectiveCPreprocessorParser#preprocessorConditional.
    def enterPreprocessorConditional(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCPreprocessorParser#preprocessorConditional.
    def exitPreprocessorConditional(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCPreprocessorParser#preprocessorDef.
    def enterPreprocessorDef(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCPreprocessorParser#preprocessorDef.
    def exitPreprocessorDef(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCPreprocessorParser#preprocessorPragma.
    def enterPreprocessorPragma(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCPreprocessorParser#preprocessorPragma.
    def exitPreprocessorPragma(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCPreprocessorParser#preprocessorError.
    def enterPreprocessorError(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCPreprocessorParser#preprocessorError.
    def exitPreprocessorError(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCPreprocessorParser#preprocessorWarning.
    def enterPreprocessorWarning(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCPreprocessorParser#preprocessorWarning.
    def exitPreprocessorWarning(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCPreprocessorParser#preprocessorDefine.
    def enterPreprocessorDefine(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCPreprocessorParser#preprocessorDefine.
    def exitPreprocessorDefine(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCPreprocessorParser#directiveText.
    def enterDirectiveText(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCPreprocessorParser#directiveText.
    def exitDirectiveText(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCPreprocessorParser#preprocessorParenthesis.
    def enterPreprocessorParenthesis(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCPreprocessorParser#preprocessorParenthesis.
    def exitPreprocessorParenthesis(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCPreprocessorParser#preprocessorNot.
    def enterPreprocessorNot(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCPreprocessorParser#preprocessorNot.
    def exitPreprocessorNot(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCPreprocessorParser#preprocessorBinary.
    def enterPreprocessorBinary(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCPreprocessorParser#preprocessorBinary.
    def exitPreprocessorBinary(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCPreprocessorParser#preprocessorConstant.
    def enterPreprocessorConstant(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCPreprocessorParser#preprocessorConstant.
    def exitPreprocessorConstant(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCPreprocessorParser#preprocessorConditionalSymbol.
    def enterPreprocessorConditionalSymbol(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCPreprocessorParser#preprocessorConditionalSymbol.
    def exitPreprocessorConditionalSymbol(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCPreprocessorParser#preprocessorDefined.
    def enterPreprocessorDefined(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCPreprocessorParser#preprocessorDefined.
    def exitPreprocessorDefined(self, ctx):
        pass





#!/usr/bin/env python
# -coding:utf-8
import Violation
import linecache

class ViolationDataMananger(object):
	def __init__(self):
		self.list = [];
		self.__total_files_number = 0
		self.priority1 = 0
		self.priority2 = 0
		self.priority3 = 0
		self.files_set = set() 
		

	def addViolation(self, violation):
		# startline = violation.startline
		# path = violation.path
		# violation.currCode = linecache.getline(path,startline)
		self.list.append(violation)

	def get_total_files_number(self):
		return str(self.__total_files_number)

	def set_total_files_number(self, file_num):
		self.__total_files_number = file_num

	def summary(self):

		
		for violation in self.list:
			# violation files
			self.files_set.add(violation.path)

			# priority
			priority = violation.get_priority()
			if priority == "1" :
				self.priority1 += 1
			elif priority == "2" :
				self.priority2 += 1
			elif priority == "3" :
				self.priority3 += 1


	def numberOfViolationsWithPriority(self, level):
		if level == 1:
			return str(self.priority1)
		elif level == 2:
			return str(self.priority2)
		elif level == 3:
			return str(self.priority3)
		else:
			return str(0)


	def numberOfFilesWithViolations(self):
		return str(len(self.files_set))



violationDataMananger = ViolationDataMananger()



#
# Copyright (c) 2012-2017 The ANTLR Project. All rights reserved.
# Use of this file is governed by the BSD 3-clause license that
# can be found in the LICENSE.txt file in the project root.
from QBCodeSpecs.antlr4.atn.ATNState import StarLoopEntryState

from QBCodeSpecs.antlr4.atn.ATNConfigSet import ATNConfigSet
from QBCodeSpecs.antlr4.dfa.DFAState import DFAState
from QBCodeSpecs.antlr4.error.Errors import IllegalStateException


class DFA(object):

    def __init__(self, atnStartState, decision=0):
        # From which ATN state did we create this DFA?
        self.atnStartState = atnStartState
        self.decision = decision
        # A set of all DFA states. Use {@link Map} so we can get old state back
        #  ({@link Set} only allows you to see if it's there).
        self._states = dict()
        self.s0 = None
        # {@code true} if this DFA is for a precedence decision; otherwise,
        # {@code false}. This is the backing field for {@link #isPrecedenceDfa},
        # {@link #setPrecedenceDfa}.
        self.precedenceDfa = False

        if isinstance(atnStartState, StarLoopEntryState):
            if atnStartState.isPrecedenceDecision:
                self.precedenceDfa = True
                precedenceState = DFAState(configs=ATNConfigSet())
                precedenceState.edges = []
                precedenceState.isAcceptState = False
                precedenceState.requiresFullContext = False
                self.s0 = precedenceState


    # Get the start state for a specific precedence value.
    #
    # @param precedence The current precedence.
    # @return The start state corresponding to the specified precedence, or
    # {@code null} if no start state exists for the specified precedence.
    #
    # @throws IllegalStateException if this is not a precedence DFA.
    # @see #isPrecedenceDfa()

    def getPrecedenceStartState(self, precedence):
        if not self.precedenceDfa:
            raise IllegalStateException("Only precedence DFAs may contain a precedence start state.")

        # s0.edges is never null for a precedence DFA
        if precedence < 0 or precedence >= len(self.s0.edges):
            return None
        return self.s0.edges[precedence]

    # Set the start state for a specific precedence value.
    #
    # @param precedence The current precedence.
    # @param startState The start state corresponding to the specified
    # precedence.
    #
    # @throws IllegalStateException if this is not a precedence DFA.
    # @see #isPrecedenceDfa()
    #
    def setPrecedenceStartState(self, precedence, startState):
        if not self.precedenceDfa:
            raise IllegalStateException("Only precedence DFAs may contain a precedence start state.")

        if precedence < 0:
            return

        # synchronization on s0 here is ok. when the DFA is turned into a
        # precedence DFA, s0 will be initialized once and not updated again
        # s0.edges is never null for a precedence DFA
        if precedence >= len(self.s0.edges):
            ext = [None] * (precedence + 1 - len(self.s0.edges))
            self.s0.edges.extend(ext)
        self.s0.edges[precedence] = startState
    #
    # Sets whether this is a precedence DFA. If the specified value differs
    # from the current DFA configuration, the following actions are taken;
    # otherwise no changes are made to the current DFA.
    #
    # <ul>
    # <li>The {@link #states} map is cleared</li>
    # <li>If {@code precedenceDfa} is {@code false}, the initial state
    # {@link #s0} is set to {@code null}; otherwise, it is initialized to a new
    # {@link DFAState} with an empty outgoing {@link DFAState#edges} array to
    # store the start states for individual precedence values.</li>
    # <li>The {@link #precedenceDfa} field is updated</li>
    # </ul>
    #
    # @param precedenceDfa {@code true} if this is a precedence DFA; otherwise,
    # {@code false}

    def setPrecedenceDfa(self, precedenceDfa):
        if self.precedenceDfa != precedenceDfa:
            self._states = dict()
            if precedenceDfa:
                precedenceState = DFAState(configs=ATNConfigSet())
                precedenceState.edges = []
                precedenceState.isAcceptState = False
                precedenceState.requiresFullContext = False
                self.s0 = precedenceState
            else:
                self.s0 = None
            self.precedenceDfa = precedenceDfa

    @property
    def states(self):
        return self._states

    # Return a list of all states in this DFA, ordered by state number.
    def sortedStates(self):
        return sorted(self._states.keys(), key=lambda state: state.stateNumber)

    def __str__(self):
        return unicode(self)

    def __unicode__(self):
        return self.toString(None)

    def toString(self, literalNames=None, symbolicNames=None):
        if self.s0 is None:
            return ""
        from QBCodeSpecs.antlr4.dfa.DFASerializer import DFASerializer
        serializer = DFASerializer(self, literalNames, symbolicNames)
        return unicode(serializer)

    def toLexerString(self):
        if self.s0 is None:
            return ""
        from QBCodeSpecs.antlr4.dfa.DFASerializer import LexerDFASerializer
        serializer = LexerDFASerializer(self)
        return unicode(serializer)




#!/usr/bin/env python
# -coding:utf-8
import sys
from QBCodeSpecs.objcRecoginition.ObjectiveCParserListener import *
from QBCodeSpecs.driver.ViolationDataMananger import *
from QBCodeSpecs.driver.Violation import *
import re



class ClassNameRuleListener(ObjectiveCParserListener):

	def __init__(self, violation):
		super(ClassNameRuleListener,self).__init__()
		self.violation = violation
		self.regex = self.violation.configinfo["Regex"]


	def send_violation(self, startline, startcolumn, classname):

		path = self.violation.path
		configInfo = self.violation.configinfo

		violInfo = Violation(path, configInfo)
		violInfo.startline = startline
		violInfo.startcolumn = startcolumn
		violInfo.message =  "类名 "+ "\"" + classname + "\"" + " 不符合正则 " + "\"" + self.regex + "\""
		violInfo.set_priority("2")
		violInfo.set_rulename("Name")
		violInfo.set_rulecategory("类名定义")
		violInfo.set_democode("@interface MttDataManager : NSObject")
		violInfo.set_noncompliant_code("@interface mttDataManager : NSObject \n\
@interface _MttDataManager : NSObject")
		
		violationDataMananger.addViolation(violInfo)
		
	def exitClassInterface(self, ctx):

		context = ctx.genericTypeSpecifier()

		# get class name
		identifier = context.identifier()
		print(identifier.getText())
		
		token = identifier.IDENTIFIER()
		commonToken = token.symbol
		print(commonToken)

		# line num
		print("line number:")
		print(commonToken.line)
		lineNum = commonToken.line
		
		# column num
		print("column number:")
		print(commonToken.column)
		columnNum = commonToken.column
		
		className = token.getText()

		if len(self.regex) > 0:
			if re.match(self.regex, className):
				print('ok')
			else:
				self.send_violation(lineNum, columnNum, className)
				print('failed')
		
			



# Copyright (c) 2012-2017 The ANTLR Project. All rights reserved.
# Use of this file is governed by the BSD 3-clause license that
# can be found in the LICENSE.txt file in the project root.
#/

# Represents the type of recognizer an ATN applies to.

class ATNType(object):

    LEXER = 0
    PARSER = 1




#
# Copyright (c) 2012-2017 The ANTLR Project. All rights reserved.
# Use of this file is governed by the BSD 3-clause license that
# can be found in the LICENSE.txt file in the project root.
#

from __future__ import unicode_literals
import sys

def is_leading_surrogate(code_unit):
    return 0xD800 <= code_unit <= 0xDBFF

def is_trailing_surrogate(code_unit):
    return 0xDC00 <= code_unit <= 0xDFFF

def decode_surrogate_pair(leading, trailing):
    return ((leading - 0xD800) << 10) + (trailing - 0xDC00) + 0x10000

def _from_unicode(unistr):
    return (ord(c) for c in unistr)

def _from_utf16(unistr):
    assert sys.maxunicode == 0xFFFF
    leading_surrogate = -1
    for utf16 in unistr:
        code_unit = ord(utf16)
        if leading_surrogate == -1:
            if is_leading_surrogate(code_unit):
                leading_surrogate = code_unit
            else:
                yield code_unit
        else:
            if is_trailing_surrogate(code_unit):
                # Valid surrogate pair
                code_point = decode_surrogate_pair(leading_surrogate, code_unit)
                yield code_point
                leading_surrogate = -1
            else:
                # Leading surrogate without trailing surrogate
                yield leading_surrogate
                if is_leading_surrogate(code_unit):
                    leading_surrogate = code_unit
                else:
                    yield code_point
                    leading_surrogate = -1
    # Dangling surrogate at end of input
    if leading_surrogate != -1:
        yield leading_surrogate

def _to_utf16(code_points):
    for code_point in code_points:
        if code_point <= 0xFFFF:
            yield unichr(code_point)
        else:
            base = code_point - 0x10000
            high_surrogate = (base >> 10) + 0xD800
            low_surrogate = (base & 0x3FF) + 0xDC00
            yield unichr(high_surrogate)
            yield unichr(low_surrogate)

def _to_chars(code_points):
    return (unichr(cp) for cp in code_points)

if sys.maxunicode == 0xFFFF:
    from_unicode = _from_utf16
    to_chars = _to_utf16
else:
    assert sys.maxunicode == 0x10FFFF
    from_unicode = _from_unicode
    to_chars = _to_chars

def to_unicode(code_points):
    return u''.join(to_chars(code_points))



# Generated from ObjectiveCParser.g4 by ANTLR 4.7.1
from QBCodeSpecs.antlr4 import *

# This class defines a complete listener for a parse tree produced by ObjectiveCParser.
class ObjectiveCParserListener(ParseTreeListener):

    # Enter a parse tree produced by ObjectiveCParser#translationUnit.
    def enterTranslationUnit(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#translationUnit.
    def exitTranslationUnit(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#topLevelDeclaration.
    def enterTopLevelDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#topLevelDeclaration.
    def exitTopLevelDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#importDeclaration.
    def enterImportDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#importDeclaration.
    def exitImportDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#classInterface.
    def enterClassInterface(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#classInterface.
    def exitClassInterface(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#categoryInterface.
    def enterCategoryInterface(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#categoryInterface.
    def exitCategoryInterface(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#classImplementation.
    def enterClassImplementation(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#classImplementation.
    def exitClassImplementation(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#categoryImplementation.
    def enterCategoryImplementation(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#categoryImplementation.
    def exitCategoryImplementation(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#genericTypeSpecifier.
    def enterGenericTypeSpecifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#genericTypeSpecifier.
    def exitGenericTypeSpecifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#protocolDeclaration.
    def enterProtocolDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#protocolDeclaration.
    def exitProtocolDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#protocolDeclarationSection.
    def enterProtocolDeclarationSection(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#protocolDeclarationSection.
    def exitProtocolDeclarationSection(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#protocolDeclarationList.
    def enterProtocolDeclarationList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#protocolDeclarationList.
    def exitProtocolDeclarationList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#classDeclarationList.
    def enterClassDeclarationList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#classDeclarationList.
    def exitClassDeclarationList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#protocolList.
    def enterProtocolList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#protocolList.
    def exitProtocolList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#propertyDeclaration.
    def enterPropertyDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#propertyDeclaration.
    def exitPropertyDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#propertyAttributesList.
    def enterPropertyAttributesList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#propertyAttributesList.
    def exitPropertyAttributesList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#propertyAttribute.
    def enterPropertyAttribute(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#propertyAttribute.
    def exitPropertyAttribute(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#protocolName.
    def enterProtocolName(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#protocolName.
    def exitProtocolName(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#instanceVariables.
    def enterInstanceVariables(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#instanceVariables.
    def exitInstanceVariables(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#visibilitySection.
    def enterVisibilitySection(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#visibilitySection.
    def exitVisibilitySection(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#accessModifier.
    def enterAccessModifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#accessModifier.
    def exitAccessModifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#interfaceDeclarationList.
    def enterInterfaceDeclarationList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#interfaceDeclarationList.
    def exitInterfaceDeclarationList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#classMethodDeclaration.
    def enterClassMethodDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#classMethodDeclaration.
    def exitClassMethodDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#instanceMethodDeclaration.
    def enterInstanceMethodDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#instanceMethodDeclaration.
    def exitInstanceMethodDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#methodDeclaration.
    def enterMethodDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#methodDeclaration.
    def exitMethodDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#implementationDefinitionList.
    def enterImplementationDefinitionList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#implementationDefinitionList.
    def exitImplementationDefinitionList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#classMethodDefinition.
    def enterClassMethodDefinition(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#classMethodDefinition.
    def exitClassMethodDefinition(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#instanceMethodDefinition.
    def enterInstanceMethodDefinition(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#instanceMethodDefinition.
    def exitInstanceMethodDefinition(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#methodDefinition.
    def enterMethodDefinition(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#methodDefinition.
    def exitMethodDefinition(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#methodSelector.
    def enterMethodSelector(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#methodSelector.
    def exitMethodSelector(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#keywordDeclarator.
    def enterKeywordDeclarator(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#keywordDeclarator.
    def exitKeywordDeclarator(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#selector.
    def enterSelector(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#selector.
    def exitSelector(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#methodType.
    def enterMethodType(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#methodType.
    def exitMethodType(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#propertyImplementation.
    def enterPropertyImplementation(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#propertyImplementation.
    def exitPropertyImplementation(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#propertySynthesizeList.
    def enterPropertySynthesizeList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#propertySynthesizeList.
    def exitPropertySynthesizeList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#propertySynthesizeItem.
    def enterPropertySynthesizeItem(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#propertySynthesizeItem.
    def exitPropertySynthesizeItem(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#blockType.
    def enterBlockType(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#blockType.
    def exitBlockType(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#genericsSpecifier.
    def enterGenericsSpecifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#genericsSpecifier.
    def exitGenericsSpecifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#typeSpecifierWithPrefixes.
    def enterTypeSpecifierWithPrefixes(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#typeSpecifierWithPrefixes.
    def exitTypeSpecifierWithPrefixes(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#dictionaryExpression.
    def enterDictionaryExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#dictionaryExpression.
    def exitDictionaryExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#dictionaryPair.
    def enterDictionaryPair(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#dictionaryPair.
    def exitDictionaryPair(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#arrayExpression.
    def enterArrayExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#arrayExpression.
    def exitArrayExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#boxExpression.
    def enterBoxExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#boxExpression.
    def exitBoxExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#blockParameters.
    def enterBlockParameters(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#blockParameters.
    def exitBlockParameters(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#typeVariableDeclaratorOrName.
    def enterTypeVariableDeclaratorOrName(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#typeVariableDeclaratorOrName.
    def exitTypeVariableDeclaratorOrName(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#blockExpression.
    def enterBlockExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#blockExpression.
    def exitBlockExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#messageExpression.
    def enterMessageExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#messageExpression.
    def exitMessageExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#receiver.
    def enterReceiver(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#receiver.
    def exitReceiver(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#messageSelector.
    def enterMessageSelector(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#messageSelector.
    def exitMessageSelector(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#keywordArgument.
    def enterKeywordArgument(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#keywordArgument.
    def exitKeywordArgument(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#keywordArgumentType.
    def enterKeywordArgumentType(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#keywordArgumentType.
    def exitKeywordArgumentType(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#selectorExpression.
    def enterSelectorExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#selectorExpression.
    def exitSelectorExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#selectorName.
    def enterSelectorName(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#selectorName.
    def exitSelectorName(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#protocolExpression.
    def enterProtocolExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#protocolExpression.
    def exitProtocolExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#encodeExpression.
    def enterEncodeExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#encodeExpression.
    def exitEncodeExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#typeVariableDeclarator.
    def enterTypeVariableDeclarator(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#typeVariableDeclarator.
    def exitTypeVariableDeclarator(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#throwStatement.
    def enterThrowStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#throwStatement.
    def exitThrowStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#tryBlock.
    def enterTryBlock(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#tryBlock.
    def exitTryBlock(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#catchStatement.
    def enterCatchStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#catchStatement.
    def exitCatchStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#synchronizedStatement.
    def enterSynchronizedStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#synchronizedStatement.
    def exitSynchronizedStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#autoreleaseStatement.
    def enterAutoreleaseStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#autoreleaseStatement.
    def exitAutoreleaseStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#functionDeclaration.
    def enterFunctionDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#functionDeclaration.
    def exitFunctionDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#functionDefinition.
    def enterFunctionDefinition(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#functionDefinition.
    def exitFunctionDefinition(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#functionSignature.
    def enterFunctionSignature(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#functionSignature.
    def exitFunctionSignature(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#attribute.
    def enterAttribute(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#attribute.
    def exitAttribute(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#attributeName.
    def enterAttributeName(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#attributeName.
    def exitAttributeName(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#attributeParameters.
    def enterAttributeParameters(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#attributeParameters.
    def exitAttributeParameters(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#attributeParameterList.
    def enterAttributeParameterList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#attributeParameterList.
    def exitAttributeParameterList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#attributeParameter.
    def enterAttributeParameter(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#attributeParameter.
    def exitAttributeParameter(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#attributeParameterAssignment.
    def enterAttributeParameterAssignment(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#attributeParameterAssignment.
    def exitAttributeParameterAssignment(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#declaration.
    def enterDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#declaration.
    def exitDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#functionCallExpression.
    def enterFunctionCallExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#functionCallExpression.
    def exitFunctionCallExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#enumDeclaration.
    def enterEnumDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#enumDeclaration.
    def exitEnumDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#varDeclaration.
    def enterVarDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#varDeclaration.
    def exitVarDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#typedefDeclaration.
    def enterTypedefDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#typedefDeclaration.
    def exitTypedefDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#typeDeclaratorList.
    def enterTypeDeclaratorList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#typeDeclaratorList.
    def exitTypeDeclaratorList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#typeDeclarator.
    def enterTypeDeclarator(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#typeDeclarator.
    def exitTypeDeclarator(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#declarationSpecifiers.
    def enterDeclarationSpecifiers(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#declarationSpecifiers.
    def exitDeclarationSpecifiers(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#attributeSpecifier.
    def enterAttributeSpecifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#attributeSpecifier.
    def exitAttributeSpecifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#initDeclaratorList.
    def enterInitDeclaratorList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#initDeclaratorList.
    def exitInitDeclaratorList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#initDeclarator.
    def enterInitDeclarator(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#initDeclarator.
    def exitInitDeclarator(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#structOrUnionSpecifier.
    def enterStructOrUnionSpecifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#structOrUnionSpecifier.
    def exitStructOrUnionSpecifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#fieldDeclaration.
    def enterFieldDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#fieldDeclaration.
    def exitFieldDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#specifierQualifierList.
    def enterSpecifierQualifierList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#specifierQualifierList.
    def exitSpecifierQualifierList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#ibOutletQualifier.
    def enterIbOutletQualifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#ibOutletQualifier.
    def exitIbOutletQualifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#arcBehaviourSpecifier.
    def enterArcBehaviourSpecifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#arcBehaviourSpecifier.
    def exitArcBehaviourSpecifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#nullabilitySpecifier.
    def enterNullabilitySpecifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#nullabilitySpecifier.
    def exitNullabilitySpecifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#storageClassSpecifier.
    def enterStorageClassSpecifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#storageClassSpecifier.
    def exitStorageClassSpecifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#typePrefix.
    def enterTypePrefix(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#typePrefix.
    def exitTypePrefix(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#typeQualifier.
    def enterTypeQualifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#typeQualifier.
    def exitTypeQualifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#protocolQualifier.
    def enterProtocolQualifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#protocolQualifier.
    def exitProtocolQualifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#typeSpecifier.
    def enterTypeSpecifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#typeSpecifier.
    def exitTypeSpecifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#typeofExpression.
    def enterTypeofExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#typeofExpression.
    def exitTypeofExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#fieldDeclaratorList.
    def enterFieldDeclaratorList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#fieldDeclaratorList.
    def exitFieldDeclaratorList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#fieldDeclarator.
    def enterFieldDeclarator(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#fieldDeclarator.
    def exitFieldDeclarator(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#enumSpecifier.
    def enterEnumSpecifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#enumSpecifier.
    def exitEnumSpecifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#enumeratorList.
    def enterEnumeratorList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#enumeratorList.
    def exitEnumeratorList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#enumerator.
    def enterEnumerator(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#enumerator.
    def exitEnumerator(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#enumeratorIdentifier.
    def enterEnumeratorIdentifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#enumeratorIdentifier.
    def exitEnumeratorIdentifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#directDeclarator.
    def enterDirectDeclarator(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#directDeclarator.
    def exitDirectDeclarator(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#declaratorSuffix.
    def enterDeclaratorSuffix(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#declaratorSuffix.
    def exitDeclaratorSuffix(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#parameterList.
    def enterParameterList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#parameterList.
    def exitParameterList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#pointer.
    def enterPointer(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#pointer.
    def exitPointer(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#macro.
    def enterMacro(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#macro.
    def exitMacro(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#arrayInitializer.
    def enterArrayInitializer(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#arrayInitializer.
    def exitArrayInitializer(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#structInitializer.
    def enterStructInitializer(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#structInitializer.
    def exitStructInitializer(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#initializerList.
    def enterInitializerList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#initializerList.
    def exitInitializerList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#typeName.
    def enterTypeName(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#typeName.
    def exitTypeName(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#abstractDeclarator.
    def enterAbstractDeclarator(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#abstractDeclarator.
    def exitAbstractDeclarator(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#abstractDeclaratorSuffix.
    def enterAbstractDeclaratorSuffix(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#abstractDeclaratorSuffix.
    def exitAbstractDeclaratorSuffix(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#parameterDeclarationList.
    def enterParameterDeclarationList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#parameterDeclarationList.
    def exitParameterDeclarationList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#parameterDeclaration.
    def enterParameterDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#parameterDeclaration.
    def exitParameterDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#declarator.
    def enterDeclarator(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#declarator.
    def exitDeclarator(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#statement.
    def enterStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#statement.
    def exitStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#labeledStatement.
    def enterLabeledStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#labeledStatement.
    def exitLabeledStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#rangeExpression.
    def enterRangeExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#rangeExpression.
    def exitRangeExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#compoundStatement.
    def enterCompoundStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#compoundStatement.
    def exitCompoundStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#selectionStatement.
    def enterSelectionStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#selectionStatement.
    def exitSelectionStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#switchStatement.
    def enterSwitchStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#switchStatement.
    def exitSwitchStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#switchBlock.
    def enterSwitchBlock(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#switchBlock.
    def exitSwitchBlock(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#switchSection.
    def enterSwitchSection(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#switchSection.
    def exitSwitchSection(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#switchLabel.
    def enterSwitchLabel(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#switchLabel.
    def exitSwitchLabel(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#iterationStatement.
    def enterIterationStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#iterationStatement.
    def exitIterationStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#whileStatement.
    def enterWhileStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#whileStatement.
    def exitWhileStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#doStatement.
    def enterDoStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#doStatement.
    def exitDoStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#forStatement.
    def enterForStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#forStatement.
    def exitForStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#forLoopInitializer.
    def enterForLoopInitializer(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#forLoopInitializer.
    def exitForLoopInitializer(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#forInStatement.
    def enterForInStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#forInStatement.
    def exitForInStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#jumpStatement.
    def enterJumpStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#jumpStatement.
    def exitJumpStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#expressions.
    def enterExpressions(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#expressions.
    def exitExpressions(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#expression.
    def enterExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#expression.
    def exitExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#assignmentOperator.
    def enterAssignmentOperator(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#assignmentOperator.
    def exitAssignmentOperator(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#castExpression.
    def enterCastExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#castExpression.
    def exitCastExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#initializer.
    def enterInitializer(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#initializer.
    def exitInitializer(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#constantExpression.
    def enterConstantExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#constantExpression.
    def exitConstantExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#unaryExpression.
    def enterUnaryExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#unaryExpression.
    def exitUnaryExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#unaryOperator.
    def enterUnaryOperator(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#unaryOperator.
    def exitUnaryOperator(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#postfixExpression.
    def enterPostfixExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#postfixExpression.
    def exitPostfixExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#postfix.
    def enterPostfix(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#postfix.
    def exitPostfix(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#argumentExpressionList.
    def enterArgumentExpressionList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#argumentExpressionList.
    def exitArgumentExpressionList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#argumentExpression.
    def enterArgumentExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#argumentExpression.
    def exitArgumentExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#primaryExpression.
    def enterPrimaryExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#primaryExpression.
    def exitPrimaryExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#constant.
    def enterConstant(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#constant.
    def exitConstant(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#stringLiteral.
    def enterStringLiteral(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#stringLiteral.
    def exitStringLiteral(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#identifier.
    def enterIdentifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#identifier.
    def exitIdentifier(self, ctx):
        pass





#
# Copyright (c) 2012-2017 The ANTLR Project. All rights reserved.
# Use of this file is governed by the BSD 3-clause license that
# can be found in the LICENSE.txt file in the project root.

# This implementation of {@link TokenStream} loads tokens from a
# {@link TokenSource} on-demand, and places the tokens in a buffer to provide
# access to any previous token by index.
#
# <p>
# This token stream ignores the value of {@link Token#getChannel}. If your
# parser requires the token stream filter tokens to only those on a particular
# channel, such as {@link Token#DEFAULT_CHANNEL} or
# {@link Token#HIDDEN_CHANNEL}, use a filtering token stream such a
# {@link CommonTokenStream}.</p>
from io import StringIO

from QBCodeSpecs.antlr4.Token import Token
from QBCodeSpecs.antlr4.error.Errors import IllegalStateException

# this is just to keep meaningful parameter types to Parser
class TokenStream(object):

    pass


class BufferedTokenStream(TokenStream):

    def __init__(self, tokenSource):
        # The {@link TokenSource} from which tokens for this stream are fetched.
        self.tokenSource = tokenSource

        # A collection of all tokens fetched from the token source. The list is
        # considered a complete view of the input once {@link #fetchedEOF} is set
        # to {@code true}.
        self.tokens = []

        # The index into {@link #tokens} of the current token (next token to
        # {@link #consume}). {@link #tokens}{@code [}{@link #p}{@code ]} should be
        # {@link #LT LT(1)}.
        #
        # <p>This field is set to -1 when the stream is first constructed or when
        # {@link #setTokenSource} is called, indicating that the first token has
        # not yet been fetched from the token source. For additional information,
        # see the documentation of {@link IntStream} for a description of
        # Initializing Methods.</p>
        self.index = -1

        # Indicates whether the {@link Token#EOF} token has been fetched from
        # {@link #tokenSource} and added to {@link #tokens}. This field improves
        # performance for the following cases:
        #
        # <ul>
        # <li>{@link #consume}: The lookahead check in {@link #consume} to prevent
        # consuming the EOF symbol is optimized by checking the values of
        # {@link #fetchedEOF} and {@link #p} instead of calling {@link #LA}.</li>
        # <li>{@link #fetch}: The check to prevent adding multiple EOF symbols into
        # {@link #tokens} is trivial with this field.</li>
        # <ul>
        self.fetchedEOF = False

    def mark(self):
        return 0

    def release(self, marker):
        # no resources to release
        pass

    def reset(self):
        self.seek(0)

    def seek(self, index):
        self.lazyInit()
        self.index = self.adjustSeekIndex(index)

    def get(self, index):
        self.lazyInit()
        return self.tokens[index]

    def consume(self):
        skipEofCheck = False
        if self.index >= 0:
            if self.fetchedEOF:
                # the last token in tokens is EOF. skip check if p indexes any
                # fetched token except the last.
                skipEofCheck = self.index < len(self.tokens) - 1
            else:
                # no EOF token in tokens. skip check if p indexes a fetched token.
                skipEofCheck = self.index < len(self.tokens)
        else:
            # not yet initialized
            skipEofCheck = False

        if not skipEofCheck and self.LA(1) == Token.EOF:
            raise IllegalStateException("cannot consume EOF")

        if self.sync(self.index + 1):
            self.index = self.adjustSeekIndex(self.index + 1)

    # Make sure index {@code i} in tokens has a token.
    #
    # @return {@code true} if a token is located at index {@code i}, otherwise
    #    {@code false}.
    # @see #get(int i)
    #/
    def sync(self, i):
        n = i - len(self.tokens) + 1 # how many more elements we need?
        if n > 0 :
            fetched = self.fetch(n)
            return fetched >= n
        return True

    # Add {@code n} elements to buffer.
    #
    # @return The actual number of elements added to the buffer.
    #/
    def fetch(self, n):
        if self.fetchedEOF:
            return 0
        for i in range(0, n):
            t = self.tokenSource.nextToken()
            t.tokenIndex = len(self.tokens)
            self.tokens.append(t)
            if t.type==Token.EOF:
                self.fetchedEOF = True
                return i + 1
        return n


    # Get all tokens from start..stop inclusively#/
    def getTokens(self, start, stop, types=None):
        if start<0 or stop<0:
            return None
        self.lazyInit()
        subset = []
        if stop >= len(self.tokens):
            stop = len(self.tokens)-1
        for i in range(start, stop):
            t = self.tokens[i]
            if t.type==Token.EOF:
                break
            if types is None or t.type in types:
                subset.append(t)
        return subset

    def LA(self, i):
        return self.LT(i).type

    def LB(self, k):
        if (self.index-k) < 0:
            return None
        return self.tokens[self.index-k]

    def LT(self, k):
        self.lazyInit()
        if k==0:
            return None
        if k < 0:
            return self.LB(-k)
        i = self.index + k - 1
        self.sync(i)
        if i >= len(self.tokens): # return EOF token
            # EOF must be last token
            return self.tokens[len(self.tokens)-1]
        return self.tokens[i]

    # Allowed derived classes to modify the behavior of operations which change
    # the current stream position by adjusting the target token index of a seek
    # operation. The default implementation simply returns {@code i}. If an
    # exception is thrown in this method, the current stream index should not be
    # changed.
    #
    # <p>For example, {@link CommonTokenStream} overrides this method to ensure that
    # the seek target is always an on-channel token.</p>
    #
    # @param i The target token index.
    # @return The adjusted target token index.

    def adjustSeekIndex(self, i):
        return i

    def lazyInit(self):
        if self.index == -1:
            self.setup()

    def setup(self):
        self.sync(0)
        self.index = self.adjustSeekIndex(0)

    # Reset this token stream by setting its token source.#/
    def setTokenSource(self, tokenSource):
        self.tokenSource = tokenSource
        self.tokens = []
        self.index = -1
        self.fetchedEOF = False


    # Given a starting index, return the index of the next token on channel.
    #  Return i if tokens[i] is on channel.  Return -1 if there are no tokens
    #  on channel between i and EOF.
    #/
    def nextTokenOnChannel(self, i, channel):
        self.sync(i)
        if i>=len(self.tokens):
            return -1
        token = self.tokens[i]
        while token.channel!=channel:
            if token.type==Token.EOF:
                return -1
            i += 1
            self.sync(i)
            token = self.tokens[i]
        return i

    # Given a starting index, return the index of the previous token on channel.
    #  Return i if tokens[i] is on channel. Return -1 if there are no tokens
    #  on channel between i and 0.
    def previousTokenOnChannel(self, i, channel):
        while i>=0 and self.tokens[i].channel!=channel:
            i -= 1
        return i

    # Collect all tokens on specified channel to the right of
    #  the current token up until we see a token on DEFAULT_TOKEN_CHANNEL or
    #  EOF. If channel is -1, find any non default channel token.
    def getHiddenTokensToRight(self, tokenIndex, channel=-1):
        self.lazyInit()
        if tokenIndex<0 or tokenIndex>=len(self.tokens):
            raise Exception(str(tokenIndex) + " not in 0.." + str(len(self.tokens)-1))
        from QBCodeSpecs.antlr4.Lexer import Lexer
        nextOnChannel = self.nextTokenOnChannel(tokenIndex + 1, Lexer.DEFAULT_TOKEN_CHANNEL)
        from_ = tokenIndex+1
        # if none onchannel to right, nextOnChannel=-1 so set to = last token
        to = (len(self.tokens)-1) if nextOnChannel==-1 else nextOnChannel
        return self.filterForChannel(from_, to, channel)


    # Collect all tokens on specified channel to the left of
    #  the current token up until we see a token on DEFAULT_TOKEN_CHANNEL.
    #  If channel is -1, find any non default channel token.
    def getHiddenTokensToLeft(self, tokenIndex, channel=-1):
        self.lazyInit()
        if tokenIndex<0 or tokenIndex>=len(self.tokens):
            raise Exception(str(tokenIndex) + " not in 0.." + str(len(self.tokens)-1))
        from QBCodeSpecs.antlr4.Lexer import Lexer
        prevOnChannel = self.previousTokenOnChannel(tokenIndex - 1, Lexer.DEFAULT_TOKEN_CHANNEL)
        if prevOnChannel == tokenIndex - 1:
            return None
        # if none on channel to left, prevOnChannel=-1 then from=0
        from_ = prevOnChannel+1
        to = tokenIndex-1
        return self.filterForChannel(from_, to, channel)


    def filterForChannel(self, left, right, channel):
        hidden = []
        for i in range(left, right+1):
            t = self.tokens[i]
            if channel==-1:
                from QBCodeSpecs.antlr4.Lexer import Lexer
                if t.channel!= Lexer.DEFAULT_TOKEN_CHANNEL:
                    hidden.append(t)
            elif t.channel==channel:
                    hidden.append(t)
        if len(hidden)==0:
            return None
        return hidden

    def getSourceName(self):
        return self.tokenSource.getSourceName()

    def getText(self, interval=None):
        """
        Get the text of all tokens in this buffer.

        :param interval:
        :type interval: antlr4.IntervalSet.Interval
        :return: string
        """
        self.lazyInit()
        self.fill()
        if interval is None:
            interval = (0, len(self.tokens)-1)
        start = interval[0]
        if isinstance(start, Token):
            start = start.tokenIndex
        stop = interval[1]
        if isinstance(stop, Token):
            stop = stop.tokenIndex
        if start is None or stop is None or start<0 or stop<0:
            return ""
        if stop >= len(self.tokens):
            stop = len(self.tokens)-1
        with StringIO() as buf:
            for i in range(start, stop+1):
                t = self.tokens[i]
                if t.type==Token.EOF:
                    break
                buf.write(t.text)
            return buf.getvalue()

    def fill(self):
        """
        Get all tokens from lexer until EOF

        :return: None
        """
        self.lazyInit()
        while self.fetch(1000)==1000:
            pass



#
# Copyright (c) 2012-2017 The ANTLR Project. All rights reserved.
# Use of this file is governed by the BSD 3-clause license that
# can be found in the LICENSE.txt file in the project root.
#/

#
# This class extends {@link BufferedTokenStream} with functionality to filter
# token streams to tokens on a particular channel (tokens where
# {@link Token#getChannel} returns a particular value).
#
# <p>
# This token stream provides access to all tokens by index or when calling
# methods like {@link #getText}. The channel filtering is only used for code
# accessing tokens via the lookahead methods {@link #LA}, {@link #LT}, and
# {@link #LB}.</p>
#
# <p>
# By default, tokens are placed on the default channel
# ({@link Token#DEFAULT_CHANNEL}), but may be reassigned by using the
# {@code ->channel(HIDDEN)} lexer command, or by using an embedded action to
# call {@link Lexer#setChannel}.
# </p>
#
# <p>
# Note: lexer rules which use the {@code ->skip} lexer command or call
# {@link Lexer#skip} do not produce tokens at all, so input text matched by
# such a rule will not be available as part of the token stream, regardless of
# channel.</p>
#/

from QBCodeSpecs.antlr4.BufferedTokenStream import BufferedTokenStream
from QBCodeSpecs.antlr4.Token import Token


class CommonTokenStream(BufferedTokenStream):

    def __init__(self, lexer, channel=Token.DEFAULT_CHANNEL):
        super(CommonTokenStream, self).__init__(lexer)
        self.channel = channel

    def adjustSeekIndex(self, i):
        return self.nextTokenOnChannel(i, self.channel)

    def LB(self, k):
        if k==0 or (self.index-k)<0:
            return None
        i = self.index
        n = 1
        # find k good tokens looking backwards
        while n <= k:
            # skip off-channel tokens
            i = self.previousTokenOnChannel(i - 1, self.channel)
            n += 1
        if i < 0:
            return None
        return self.tokens[i]

    def LT(self, k):
        self.lazyInit()
        if k == 0:
            return None
        if k < 0:
            return self.LB(-k)
        i = self.index
        n = 1 # we know tokens[pos] is a good one
        # find k good tokens
        while n < k:
            # skip off-channel tokens, but make sure to not look past EOF
            if self.sync(i + 1):
                i = self.nextTokenOnChannel(i + 1, self.channel)
            n += 1
        return self.tokens[i]

    # Count EOF just once.#/
    def getNumberOfOnChannelTokens(self):
        n = 0
        self.fill()
        for i in range(0, len(self.tokens)):
            t = self.tokens[i]
            if t.channel==self.channel:
                n += 1
            if t.type==Token.EOF:
                break
        return n



#
# Copyright (c) 2012-2017 The ANTLR Project. All rights reserved.
# Use of this file is governed by the BSD 3-clause license that
# can be found in the LICENSE.txt file in the project root.
#

class Chunk(object):

    def __str__(self):
        return unicode(self)


class TagChunk(Chunk):

    def __init__(self, tag, label=None):
        self.tag = tag
        self.label = label

    def __unicode__(self):
        if self.label is None:
            return self.tag
        else:
            return self.label + ":" + self.tag

class TextChunk(Chunk):

    def __init__(self, text):
        self.text = text

    def __unicode__(self):
        return "'" + self.text + "'"




#!/usr/bin/env python
# -coding:utf-8
import io
from QBCodeSpecs.driver.ViolationDataMananger import *
from QBCodeSpecs.driver.Violation import *

class HTMLReporter(object):
	"""docstring for HTMLReporter"""


	


	def __init__(self):
		super(HTMLReporter, self).__init__()


	def write_head(self, f):
		head_string = "<head> 				 \
        <title>QBCodeTools Report</title>	 \
        <meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" /> \
        <style type='text/css'>				 \
.priority1, .priority2, .priority3,          \
.cmplr-error, .cmplr-warning, .checker-bug { \
    font-weight: bold;                       \
    text-align: center;                      \
}                                            \
.priority1, .priority2, .priority3 {         \
    color: #BF0A30;                          \
}                                            \
.priority1 { background-color: #FFC200; }    \
.priority2 { background-color: #FFD3A6; }    \
.priority3 { background-color: #FFEEB5; }    \
.cmplr-error, .cmplr-warning {               \
    background-color: #BF0A30;               \
}                                            \
.cmplr-error { color: #FFC200; }             \
.cmplr-warning { color: #FFD3A6; }           \
.checker-bug {                               \
    background-color: #002868;               \
    color: white;                            \
}                                            \
table {                                      \
    border: 2px solid gray;                  \
    border-collapse: collapse;               \
    -moz-box-shadow: 3px 3px 4px #AAA;       \
    -webkit-box-shadow: 3px 3px 4px #AAA;    \
    box-shadow: 3px 3px 4px #AAA;            \
}                                            \
td, th {                                     \
    border: 1px solid #D3D3D3;               \
    padding: 4px 20px 4px 20px;              \
}                                            \
th {                                         \
    text-shadow: 2px 2px 2px white;          \
    border-bottom: 1px solid gray;           \
    background-color: #E9F4FF;               \
} </style></head>"
		
		try:
			f.write(head_string)
		except Exception, e:
			print "write_head error"
			raise e

	def write_summary_table(self, f):
		summary = ("<table><thead><tr><th>扫描文件数</th><th>违规文件数</th>" + "<th>优先级 1</th><th>优先级 2</th><th>优先级 3</th>"
		+ "</tr></thead>"
		+ "<tbody><tr><td>" + violationDataMananger.get_total_files_number() + "</td><td>"
		+ violationDataMananger.numberOfFilesWithViolations() + "</td><td class='priority1'>"
		+ violationDataMananger.numberOfViolationsWithPriority(1) + "</td><td class='priority2'>"
		+ violationDataMananger.numberOfViolationsWithPriority(2) + "</td><td class='priority3'>"
		+ violationDataMananger.numberOfViolationsWithPriority(3) + "</tr></tbody></table>")
		
		try:
			f.write(summary)
		except Exception, e:
			print "write_summary_table error"
			raise e

	def write_detail_table(self, f, violation):
		detail = ("<tr><td>" + violation.path + "</td><td>" + str(violation.startline) + ":" + str(violation.startcolumn) + "</td>" 
		# + "<td>" + violation.get_rulename() + "</td>"
		+ "<td>" + violation.get_rulecategory() + "</td>"
		+ "<td class='priority" + violation.get_priority() + "'>"
		+ violation.get_priority() + "</td>"
		+ "<td>" + violation.message + "</td>"
		# + "<td><pre>" + violation.currCode + "<pre></td>"
		+ "<td><pre><code class=\"html\">" + violation.get_noncompliant_code() + "</code><pre></td>"
		+ "<td><pre><code class=\"html\">" + violation.get_democode() + "</code><pre></td></tr>")
		
		try:
			f.write(detail)
		except Exception, e:
			print "write_detail_table error"
			raise e

	def write_code_highlight(self, f):
		highlight = ("<link rel=\"stylesheet\" href=\"./styles/xcode.css\">"
		+ "<script src=\"./styles/highlight.pack.js\"></script>"
		+ "<script>hljs.initHighlightingOnLoad();</script>")
		try:
			f.write(highlight)
		except Exception, e:
			print "write_code_highlight error"
			raise e


	def report(self, filepath = ''):
		# filepath = '/Users/paulineli/Desktop/result.html'
		with open(filepath, 'w') as f:
			f.write("<!DOCTYPE html>")
			f.write("<html>")

			self.write_head(f)
			self.write_code_highlight(f)

			f.write("<body>")
			f.write("<h1>QBCodeSpecsTools 扫描报告</h1>")
			f.write("<hr />")
			f.write("<h2>汇总</h2>")
			
			self.write_summary_table(f)

			f.write("<hr />")
			f.write("<h2>详情</h2>")
			html_text = ("<table><thead><tr><th>文件路径</th><th>出错位置 行号:列号</th>" 
			# + "<th>Rule Name</th><th>Rule Category</th>" 
			+ "<th>规则类型</th>" 
			+ "<th>优先级</th><th>错误信息</th>"
			# + "<th>当前写法</th>"
			+ "<th>不合规范实例</th><th>规范实例</th></tr></thead><tbody>")
			f.write(html_text)
        	
			violationlist = violationDataMananger.list
			for violation in violationlist:
				self.write_detail_table(f, violation)

			html_text = ("</tbody></table>" + "<hr />" + "</body>" + "</html>")
			f.write(html_text)







#!/usr/bin/env python
# -coding:utf-8
import sys
from QBCodeSpecs.antlr4 import *
from QBCodeSpecs.objcRecoginition.ObjectiveCLexer import *
from QBCodeSpecs.objcRecoginition.ObjectiveCParser import *
# from QBCodeSpecs.rules.custom.CustomClassNameRuleListener import *
# from QBCodeSpecs.rules.custom.CustomMethodNameRuleListener import *
# from QBCodeSpecs.rules.name.ClassNameRuleListener import *
# from QBCodeSpecs.rules.statements.SingletonRuleListener import *


from QBCodeSpecs.driver.ViolationDataMananger import *
from QBCodeSpecs.driver.Violation import *

from QBCodeSpecs.reporters.HTMLReporter import *

import json
import os


import sys
reload(sys)
sys.setdefaultencoding('utf-8')

class ObjCCheckMain(object):
    
    def load_config(self):
        with open("QBCodeSpecs/config/config.json",'r') as load_f:
            load_dict = json.load(load_f)
            self.config = load_dict
            # print load_dict


    # def __init__(self, filepathStr = ""):
    #     super(ObjCCheckMain, self).__init__()
    #     filepathlist = filepathStr.split(';')
    #     self.filepathlist = filepathlist
    #     self.load_config()
    #     print len(self.filepathlist)


    def __init__(self, filepathlist = [], basepath = ""):
       
        super(ObjCCheckMain, self).__init__()
        self.filepathlist = filepathlist
        self.basepath = basepath
        self.load_config()
        # print len(self.filepathlist)


#def walkerTree(stream, listener):
#    parser = ObjectiveCParser(stream)
#    tree = parser.translationUnit()
#    walker = ParseTreeWalker()
#    walker.walk(listener, tree)
    # print("LISP:")
    # print(tree.toStringTree(parser))
             
    def print_result(self):
        print "final results:--------------------------------"

        violations = violationDataMananger.list
        for x in violations:
            if isinstance(x, Violation):
                info = "filepath:" + x.path + " " + "line number:" + str(x.startline) + " "+ "columnNum:" + str(x.startcolumn) + " " + "Message:" + x.message + " " + "DemoCode:" + x.get_democode()
                print info
        return len(violations)

    def is_valid_rule(self, ruleclassname):
        
        rulename = self.ruleNameFormClassName(ruleclassname)
        
        if self.config.has_key('invalidRules'):

            invalidRules = self.config['invalidRules']
            if rulename in invalidRules:
                return False
        
        return True

    def ruleNameFormClassName(self, ruleclassname):
        if ruleclassname.endswith('RuleListener') == False:
            return ""

        rulenamelen = len(ruleclassname) - len('RuleListener')
        if rulenamelen == 0:
            return ""

        rulename = ruleclassname[0:rulenamelen]
        return rulename

    def valid_rule_listener_class(self):
        # key: className value: Class
        valid_class = {}

        validDir = ["name", "statements", "custom"]
        for rulesDir in validDir:
            rulepath = "QBCodeSpecs/rules/" + rulesDir

            path=os.path.abspath('.')
            exploit_path=os.path.join(path,rulepath)
            sys.path.append(exploit_path)

            modules=[x for x in os.listdir(exploit_path) if os.path.isfile(os.path.join(exploit_path,x)) and os.path.splitext(x)[1]=='.py']
            for m in modules:
                if m!='__init__.py':
                    module_name=os.path.join(exploit_path,os.path.splitext(m)[0])
                    class_name = os.path.splitext(m)[0]
                    # print module_name
                    # print "[0]:" + class_name

                    if self.is_valid_rule(class_name) :
                        module=__import__(os.path.splitext(m)[0])
                        ruleLinstenerCls=getattr(module,class_name)
                        # print self.ruleNameFormClassName(class_name)
                        # print ruleLinstenerCls
                        valid_class[self.ruleNameFormClassName(class_name)] = ruleLinstenerCls
             

        # print valid_class
        return valid_class


    def check_file(self, inputstream, filepath):

        lexer = ObjectiveCLexer(inputstream)
        stream = CommonTokenStream(lexer)
        parser = ObjectiveCParser(stream)
        tree = parser.translationUnit()
        walker = ParseTreeWalker()

        # load rules
        rules = self.config['rules']
        # print rules
        
        listenerClassDict = self.valid_rule_listener_class()
        for (k,v) in  listenerClassDict.items(): 
            rulename = k
            listenerCls = v

            violation = None
            # has dict para
            if rules.has_key(rulename) :
                para = rules[rulename]
                violation = Violation(filepath, para)
            else:
                violation = Violation(filepath)

            listener = listenerCls(violation)
            walker.walk(listener, tree)


    def isIgnorePath(self, path):
        if len(path) == 0:
            return True

        if self.config.has_key('ignorePath') :
            ignorePath = self.config['ignorePath']
            for iPath in ignorePath:
                # print "iPath:" + iPath
                # print "path:" + path
                if path.startswith(iPath) :
                    return True
        return False

    def start_check(self):

        total_files = 0

        for filepath in self.filepathlist:

            if self.isIgnorePath(filepath) :
                continue

            absolute_path = ""
            if len(self.basepath) > 0:
                absolute_path = self.basepath + "/" + filepath
            else:
                absolute_path = filepath


            inputstream = None
            try:
                inputstream = FileStream(absolute_path, encoding='utf-8')
                # input = FileStream(filepath)
                print "open file--------------------------------"
                print "filepath:" + filepath
            except IOError:
                print "IOError**********************************"
                print "basepath:" + self.basepath
                print "filepath:" + filepath
                continue
            else:
                pass
            finally:
                pass

            if input == None:
                continue
            # print input

            # record total_files
            total_files = total_files + 1
            self.check_file(inputstream, absolute_path)
            
        if total_files == 0 :
            print "--------------no error---------------"
            return
        else :
            print "--------------check end---------------"
        violationDataMananger.set_total_files_number(total_files)
        violationDataMananger.summary()
        # self.print_result()

        reporter = HTMLReporter()
        reporter.report('QBCodeToolsResult.html')

        error_num = len(violationDataMananger.list)
        return error_num
        
        # print "return info: " + str(len(violationDataMananger.list))
        # return len(violationDataMananger.list)


def main(argv=None):
    # fileNamesString = sys.argv[1]
    if argv is None:
        argv = sys.argv

    if len(argv) < 2:
        return 0
          
    fileNamesString = argv[1]
    base_path = ""
    if len(argv) > 2:
        base_path = argv[2]
    
    # print "fileNamesString--------"
    # print fileNamesString
    filenamelist = fileNamesString.split(';')
    checker = ObjCCheckMain(filepathlist=filenamelist, basepath=base_path)
    error_num = checker.start_check()
    if error_num > 0:
        return 1
    else:
        return 0
 
if __name__ == '__main__':

    sys.exit(main())



#!/usr/bin/env python
# -coding:utf-8
import sys
from QBCodeSpecs.antlr4 import *
from QBCodeSpecs.objcRecoginition.ObjectiveCLexer import *
from QBCodeSpecs.objcRecoginition.ObjectiveCParser import *
# from QBCodeSpecs.rules.custom.CustomClassNameRuleListener import *
# from QBCodeSpecs.rules.custom.CustomMethodNameRuleListener import *
from QBCodeSpecs.rules.name.ClassNameRuleListener import *
from QBCodeSpecs.rules.statements.SingletonRuleListener import *


from QBCodeSpecs.driver.ViolationDataMananger import *
from QBCodeSpecs.driver.Violation import *

from QBCodeSpecs.reporters.HTMLReporter import *

import json


import sys
reload(sys)
sys.setdefaultencoding('utf-8')

class ObjCCheckMain(object):
    
    def load_config(self):
        with open("QBCodeSpecs/config/config.json",'r') as load_f:
            load_dict = json.load(load_f)
            self.config = load_dict
            # print load_dict


    # def __init__(self, filepathStr = ""):
    #     super(ObjCCheckMain, self).__init__()
    #     filepathlist = filepathStr.split(';')
    #     self.filepathlist = filepathlist
    #     self.load_config()
    #     print len(self.filepathlist)


    def __init__(self, filepathlist = [], basepath = ""):
       
        super(ObjCCheckMain, self).__init__()
        self.filepathlist = filepathlist
        self.basepath = basepath
        self.load_config()
        # print len(self.filepathlist)


#def walkerTree(stream, listener):
#    parser = ObjectiveCParser(stream)
#    tree = parser.translationUnit()
#    walker = ParseTreeWalker()
#    walker.walk(listener, tree)
    # print("LISP:")
    # print(tree.toStringTree(parser))
             
    def print_result(self):
        print "final results:--------------------------------"

        violations = violationDataMananger.list
        for x in violations:
            if isinstance(x, Violation):
                info = "filepath:" + x.path + " " + "line number:" + str(x.startline) + " "+ "columnNum:" + str(x.startcolumn) + " " + "Message:" + x.message + " " + "DemoCode:" + x.get_democode()
                print info
        return len(violations)

    def check_file(self, inputstream, filepath):
        lexer = ObjectiveCLexer(inputstream)
        stream = CommonTokenStream(lexer)
        parser = ObjectiveCParser(stream)
        tree = parser.translationUnit()
        walker = ParseTreeWalker()

        # load rules
        rules = self.config['rules']
        # print rules


        # check class name
        classNameRules = rules["ClassName"]
        classNameRegex = classNameRules["Regex"]
        # print classNameRegex
        if len(classNameRegex) > 0:
            print "--------------------classNameRegex-------------------------"

            violation = Violation(filepath, classNameRules)
            # print violation
            classNameListener = ClassNameRuleListener(violation)
            # print classNameListener
            walker.walk(classNameListener, tree)
            # print walker

        print "--------------------singleton-------------------------"
        # singleton rule
        violation = Violation(filepath)
        singletonListener = SingletonRuleListener(violation)
        walker.walk(singletonListener, tree)

    def start_check(self):

        total_files = 0

        for filepath in self.filepathlist:

            if len(filepath) == 0:
                continue

            absolute_path = ""
            if len(self.basepath) > 0:
                absolute_path = self.basepath + "/" + filepath
            else:
                absolute_path = filepath


            inputstream = None
            try:
                inputstream = FileStream(absolute_path, encoding='utf-8')
                # input = FileStream(filepath)
                print "open file--------------------------------"
                print "filepath:" + filepath
            except IOError:
                print "IOError**********************************"
                print "basepath:" + self.basepath
                print "filepath:" + filepath
                continue
            else:
                pass
            finally:
                pass

            if input == None:
                continue
            # print input

            # record total_files
            total_files = total_files + 1
            self.check_file(inputstream, absolute_path)
            
        violationDataMananger.set_total_files_number(total_files)
        violationDataMananger.summary()
        # self.print_result()

        reporter = HTMLReporter()
        reporter.report('QBCodeToolsResult.html')

    
 
if __name__ == '__main__':
    
    # main(sys.argv)

    fileNamesString = sys.argv[1]

    base_path = ""
    if len(sys.argv) > 2:
        base_path = sys.argv[2]
    
    # print "fileNamesString--------"
    # print fileNamesString
    filenamelist = fileNamesString.split(';')
    checker = ObjCCheckMain(filepathlist=filenamelist, basepath=base_path)
    checker.start_check()
    
    # reporter = HTMLReporter()
    # reporter.report()



# Copyright (c) 2012-2017 The ANTLR Project. All rights reserved.
# Use of this file is governed by the BSD 3-clause license that
# can be found in the LICENSE.txt file in the project root.
#/
from uuid import UUID
from QBCodeSpecs.antlr4.atn.ATN import ATN
from QBCodeSpecs.antlr4.atn.ATNType import ATNType
from QBCodeSpecs.antlr4.atn.ATNState import *
from QBCodeSpecs.antlr4.atn.Transition import *
from QBCodeSpecs.antlr4.atn.LexerAction import *
from QBCodeSpecs.antlr4.atn.ATNDeserializationOptions import ATNDeserializationOptions

# This is the earliest supported serialized UUID.
BASE_SERIALIZED_UUID = UUID("AADB8D7E-AEEF-4415-AD2B-8204D6CF042E")

# This UUID indicates the serialized ATN contains two sets of
# IntervalSets, where the second set's values are encoded as
# 32-bit integers to support the full Unicode SMP range up to U+10FFFF.
ADDED_UNICODE_SMP = UUID("59627784-3BE5-417A-B9EB-8131A7286089")

# This list contains all of the currently supported UUIDs, ordered by when
# the feature first appeared in this branch.
SUPPORTED_UUIDS = [ BASE_SERIALIZED_UUID, ADDED_UNICODE_SMP ]

SERIALIZED_VERSION = 3

# This is the current serialized UUID.
SERIALIZED_UUID = ADDED_UNICODE_SMP

class ATNDeserializer (object):

    def __init__(self, options = None):
        if options is None:
            options = ATNDeserializationOptions.defaultOptions
        self.deserializationOptions = options
        self.edgeFactories = None
        self.stateFactories = None
        self.actionFactories = None

    # Determines if a particular serialized representation of an ATN supports
    # a particular feature, identified by the {@link UUID} used for serializing
    # the ATN at the time the feature was first introduced.
    #
    # @param feature The {@link UUID} marking the first time the feature was
    # supported in the serialized ATN.
    # @param actualUuid The {@link UUID} of the actual serialized ATN which is
    # currently being deserialized.
    # @return {@code true} if the {@code actualUuid} value represents a
    # serialized ATN at or after the feature identified by {@code feature} was
    # introduced; otherwise, {@code false}.

    def isFeatureSupported(self, feature, actualUuid):
        idx1 = SUPPORTED_UUIDS.index(feature)
        if idx1<0:
            return False
        idx2 = SUPPORTED_UUIDS.index(actualUuid)
        return idx2 >= idx1

    def deserialize(self, data):
        self.reset(data)
        self.checkVersion()
        self.checkUUID()
        atn = self.readATN()
        self.readStates(atn)
        self.readRules(atn)
        self.readModes(atn)
        sets = []
        # First, read all sets with 16-bit Unicode code points <= U+FFFF.
        self.readSets(atn, sets, self.readInt)
        # Next, if the ATN was serialized with the Unicode SMP feature,
        # deserialize sets with 32-bit arguments <= U+10FFFF.
        if self.isFeatureSupported(ADDED_UNICODE_SMP, self.uuid):
            self.readSets(atn, sets, self.readInt32)
        self.readEdges(atn, sets)
        self.readDecisions(atn)
        self.readLexerActions(atn)
        self.markPrecedenceDecisions(atn)
        self.verifyATN(atn)
        if self.deserializationOptions.generateRuleBypassTransitions \
                and atn.grammarType == ATNType.PARSER:
            self.generateRuleBypassTransitions(atn)
            # re-verify after modification
            self.verifyATN(atn)
        return atn

    def reset(self, data):
        def adjust(c):
            v = ord(c)
            return v-2 if v>1 else v + 65533
        temp = [ adjust(c) for c in data ]
        # don't adjust the first value since that's the version number
        temp[0] = ord(data[0])
        self.data = temp
        self.pos = 0

    def checkVersion(self):
        version = self.readInt()
        if version != SERIALIZED_VERSION:
            raise Exception("Could not deserialize ATN with version " + str(version) + " (expected " + str(SERIALIZED_VERSION) + ").")

    def checkUUID(self):
        uuid = self.readUUID()
        if not uuid in SUPPORTED_UUIDS:
            raise Exception("Could not deserialize ATN with UUID: " + str(uuid) + \
                            " (expected " + str(SERIALIZED_UUID) + " or a legacy UUID).", uuid, SERIALIZED_UUID)
        self.uuid = uuid

    def readATN(self):
        grammarType = self.readInt()
        maxTokenType = self.readInt()
        return ATN(grammarType, maxTokenType)

    def readStates(self, atn):
        loopBackStateNumbers = []
        endStateNumbers = []
        nstates = self.readInt()
        for i in range(0, nstates):
            stype = self.readInt()
            # ignore bad type of states
            if stype==ATNState.INVALID_TYPE:
                atn.addState(None)
                continue
            ruleIndex = self.readInt()
            if ruleIndex == 0xFFFF:
                ruleIndex = -1

            s = self.stateFactory(stype, ruleIndex)
            if stype == ATNState.LOOP_END: # special case
                loopBackStateNumber = self.readInt()
                loopBackStateNumbers.append((s, loopBackStateNumber))
            elif isinstance(s, BlockStartState):
                endStateNumber = self.readInt()
                endStateNumbers.append((s, endStateNumber))

            atn.addState(s)

        # delay the assignment of loop back and end states until we know all the state instances have been initialized
        for pair in loopBackStateNumbers:
            pair[0].loopBackState = atn.states[pair[1]]

        for pair in endStateNumbers:
            pair[0].endState = atn.states[pair[1]]

        numNonGreedyStates = self.readInt()
        for i in range(0, numNonGreedyStates):
            stateNumber = self.readInt()
            atn.states[stateNumber].nonGreedy = True

        numPrecedenceStates = self.readInt()
        for i in range(0, numPrecedenceStates):
            stateNumber = self.readInt()
            atn.states[stateNumber].isPrecedenceRule = True

    def readRules(self, atn):
        nrules = self.readInt()
        if atn.grammarType == ATNType.LEXER:
            atn.ruleToTokenType = [0] * nrules

        atn.ruleToStartState = [0] * nrules
        for i in range(0, nrules):
            s = self.readInt()
            startState = atn.states[s]
            atn.ruleToStartState[i] = startState
            if atn.grammarType == ATNType.LEXER:
                tokenType = self.readInt()
                if tokenType == 0xFFFF:
                    tokenType = Token.EOF

                atn.ruleToTokenType[i] = tokenType

        atn.ruleToStopState = [0] * nrules
        for state in atn.states:
            if not isinstance(state, RuleStopState):
                continue
            atn.ruleToStopState[state.ruleIndex] = state
            atn.ruleToStartState[state.ruleIndex].stopState = state

    def readModes(self, atn):
        nmodes = self.readInt()
        for i in range(0, nmodes):
            s = self.readInt()
            atn.modeToStartState.append(atn.states[s])

    def readSets(self, atn, sets, readUnicode):
        m = self.readInt()
        for i in range(0, m):
            iset = IntervalSet()
            sets.append(iset)
            n = self.readInt()
            containsEof = self.readInt()
            if containsEof!=0:
                iset.addOne(-1)
            for j in range(0, n):
                i1 = readUnicode()
                i2 = readUnicode()
                iset.addRange(Interval(i1, i2 + 1)) # range upper limit is exclusive

    def readEdges(self, atn, sets):
        nedges = self.readInt()
        for i in range(0, nedges):
            src = self.readInt()
            trg = self.readInt()
            ttype = self.readInt()
            arg1 = self.readInt()
            arg2 = self.readInt()
            arg3 = self.readInt()
            trans = self.edgeFactory(atn, ttype, src, trg, arg1, arg2, arg3, sets)
            srcState = atn.states[src]
            srcState.addTransition(trans)

        # edges for rule stop states can be derived, so they aren't serialized
        for state in atn.states:
            for i in range(0, len(state.transitions)):
                t = state.transitions[i]
                if not isinstance(t, RuleTransition):
                    continue
                outermostPrecedenceReturn = -1
                if atn.ruleToStartState[t.target.ruleIndex].isPrecedenceRule:
                    if t.precedence == 0:
                        outermostPrecedenceReturn = t.target.ruleIndex
                trans = EpsilonTransition(t.followState, outermostPrecedenceReturn)
                atn.ruleToStopState[t.target.ruleIndex].addTransition(trans)

        for state in atn.states:
            if isinstance(state, BlockStartState):
                # we need to know the end state to set its start state
                if state.endState is None:
                    raise Exception("IllegalState")
                # block end states can only be associated to a single block start state
                if state.endState.startState is not None:
                    raise Exception("IllegalState")
                state.endState.startState = state

            elif isinstance(state, PlusLoopbackState):
                for i in range(0, len(state.transitions)):
                    target = state.transitions[i].target
                    if isinstance(target, PlusBlockStartState):
                        target.loopBackState = state
            elif isinstance(state, StarLoopbackState):
                for i in range(0, len(state.transitions)):
                    target = state.transitions[i].target
                    if isinstance(target, StarLoopEntryState):
                        target.loopBackState = state

    def readDecisions(self, atn):
        ndecisions = self.readInt()
        for i in range(0, ndecisions):
            s = self.readInt()
            decState = atn.states[s]
            atn.decisionToState.append(decState)
            decState.decision = i

    def readLexerActions(self, atn):
        if atn.grammarType == ATNType.LEXER:
            count = self.readInt()
            atn.lexerActions = [ None ] * count
            for i in range(0, count):
                actionType = self.readInt()
                data1 = self.readInt()
                if data1 == 0xFFFF:
                    data1 = -1
                data2 = self.readInt()
                if data2 == 0xFFFF:
                    data2 = -1
                lexerAction = self.lexerActionFactory(actionType, data1, data2)
                atn.lexerActions[i] = lexerAction

    def generateRuleBypassTransitions(self, atn):

        count = len(atn.ruleToStartState)
        atn.ruleToTokenType = [ 0 ] * count
        for i in range(0, count):
            atn.ruleToTokenType[i] = atn.maxTokenType + i + 1

        for i in range(0, count):
            self.generateRuleBypassTransition(atn, i)

    def generateRuleBypassTransition(self, atn, idx):

        bypassStart = BasicBlockStartState()
        bypassStart.ruleIndex = idx
        atn.addState(bypassStart)

        bypassStop = BlockEndState()
        bypassStop.ruleIndex = idx
        atn.addState(bypassStop)

        bypassStart.endState = bypassStop
        atn.defineDecisionState(bypassStart)

        bypassStop.startState = bypassStart

        excludeTransition = None

        if atn.ruleToStartState[idx].isPrecedenceRule:
            # wrap from the beginning of the rule to the StarLoopEntryState
            endState = None
            for state in atn.states:
                if self.stateIsEndStateFor(state, idx):
                    endState = state
                    excludeTransition = state.loopBackState.transitions[0]
                    break

            if excludeTransition is None:
                raise Exception("Couldn't identify final state of the precedence rule prefix section.")

        else:

            endState = atn.ruleToStopState[idx]

        # all non-excluded transitions that currently target end state need to target blockEnd instead
        for state in atn.states:
            for transition in state.transitions:
                if transition == excludeTransition:
                    continue
                if transition.target == endState:
                    transition.target = bypassStop

        # all transitions leaving the rule start state need to leave blockStart instead
        ruleToStartState = atn.ruleToStartState[idx]
        count = len(ruleToStartState.transitions)
        while count > 0:
            bypassStart.addTransition(ruleToStartState.transitions[count-1])
            del ruleToStartState.transitions[-1]

        # link the new states
        atn.ruleToStartState[idx].addTransition(EpsilonTransition(bypassStart))
        bypassStop.addTransition(EpsilonTransition(endState))

        matchState = BasicState()
        atn.addState(matchState)
        matchState.addTransition(AtomTransition(bypassStop, atn.ruleToTokenType[idx]))
        bypassStart.addTransition(EpsilonTransition(matchState))


    def stateIsEndStateFor(self, state, idx):
        if state.ruleIndex != idx:
            return None
        if not isinstance(state, StarLoopEntryState):
            return None

        maybeLoopEndState = state.transitions[len(state.transitions) - 1].target
        if not isinstance(maybeLoopEndState, LoopEndState):
            return None

        if maybeLoopEndState.epsilonOnlyTransitions and \
                isinstance(maybeLoopEndState.transitions[0].target, RuleStopState):
            return state
        else:
            return None


    #
    # Analyze the {@link StarLoopEntryState} states in the specified ATN to set
    # the {@link StarLoopEntryState#isPrecedenceDecision} field to the
    # correct value.
    #
    # @param atn The ATN.
    #
    def markPrecedenceDecisions(self, atn):
        for state in atn.states:
            if not isinstance(state, StarLoopEntryState):
                continue

            # We analyze the ATN to determine if this ATN decision state is the
            # decision for the closure block that determines whether a
            # precedence rule should continue or complete.
            #
            if atn.ruleToStartState[state.ruleIndex].isPrecedenceRule:
                maybeLoopEndState = state.transitions[len(state.transitions) - 1].target
                if isinstance(maybeLoopEndState, LoopEndState):
                    if maybeLoopEndState.epsilonOnlyTransitions and \
                            isinstance(maybeLoopEndState.transitions[0].target, RuleStopState):
                        state.isPrecedenceDecision = True

    def verifyATN(self, atn):
        if not self.deserializationOptions.verifyATN:
            return
        # verify assumptions
        for state in atn.states:
            if state is None:
                continue

            self.checkCondition(state.epsilonOnlyTransitions or len(state.transitions) <= 1)

            if isinstance(state, PlusBlockStartState):
                self.checkCondition(state.loopBackState is not None)

            if isinstance(state, StarLoopEntryState):
                self.checkCondition(state.loopBackState is not None)
                self.checkCondition(len(state.transitions) == 2)

                if isinstance(state.transitions[0].target, StarBlockStartState):
                    self.checkCondition(isinstance(state.transitions[1].target, LoopEndState))
                    self.checkCondition(not state.nonGreedy)
                elif isinstance(state.transitions[0].target, LoopEndState):
                    self.checkCondition(isinstance(state.transitions[1].target, StarBlockStartState))
                    self.checkCondition(state.nonGreedy)
                else:
                    raise Exception("IllegalState")

            if isinstance(state, StarLoopbackState):
                self.checkCondition(len(state.transitions) == 1)
                self.checkCondition(isinstance(state.transitions[0].target, StarLoopEntryState))

            if isinstance(state, LoopEndState):
                self.checkCondition(state.loopBackState is not None)

            if isinstance(state, RuleStartState):
                self.checkCondition(state.stopState is not None)

            if isinstance(state, BlockStartState):
                self.checkCondition(state.endState is not None)

            if isinstance(state, BlockEndState):
                self.checkCondition(state.startState is not None)

            if isinstance(state, DecisionState):
                self.checkCondition(len(state.transitions) <= 1 or state.decision >= 0)
            else:
                self.checkCondition(len(state.transitions) <= 1 or isinstance(state, RuleStopState))

    def checkCondition(self, condition, message=None):
        if not condition:
            if message is None:
                message = "IllegalState"
            raise Exception(message)

    def readInt(self):
        i = self.data[self.pos]
        self.pos += 1
        return i

    def readInt32(self):
        low = self.readInt()
        high = self.readInt()
        return low | (high << 16)

    def readLong(self):
        low = self.readInt32()
        high = self.readInt32()
        return (low & 0x00000000FFFFFFFF) | (high << 32)

    def readUUID(self):
        low = self.readLong()
        high = self.readLong()
        allBits = (low & 0xFFFFFFFFFFFFFFFF) | (high << 64)
        return UUID(int=allBits)

    def edgeFactory(self, atn, type, src, trg, arg1, arg2, arg3, sets):
        target = atn.states[trg]
        if self.edgeFactories is None:
            ef = [None] * 11
            ef[0] = lambda args : None
            ef[Transition.EPSILON] = lambda atn, src, trg, arg1, arg2, arg3, sets, target : \
                EpsilonTransition(target)
            ef[Transition.RANGE] = lambda atn, src, trg, arg1, arg2, arg3, sets, target : \
                RangeTransition(target, Token.EOF, arg2) if arg3 != 0 else RangeTransition(target, arg1, arg2)
            ef[Transition.RULE] = lambda atn, src, trg, arg1, arg2, arg3, sets, target : \
                RuleTransition(atn.states[arg1], arg2, arg3, target)
            ef[Transition.PREDICATE] = lambda atn, src, trg, arg1, arg2, arg3, sets, target : \
                PredicateTransition(target, arg1, arg2, arg3 != 0)
            ef[Transition.PRECEDENCE] = lambda atn, src, trg, arg1, arg2, arg3, sets, target : \
                PrecedencePredicateTransition(target, arg1)
            ef[Transition.ATOM] = lambda atn, src, trg, arg1, arg2, arg3, sets, target : \
                AtomTransition(target, Token.EOF) if arg3 != 0 else AtomTransition(target, arg1)
            ef[Transition.ACTION] = lambda atn, src, trg, arg1, arg2, arg3, sets, target : \
                ActionTransition(target, arg1, arg2, arg3 != 0)
            ef[Transition.SET] = lambda atn, src, trg, arg1, arg2, arg3, sets, target : \
                SetTransition(target, sets[arg1])
            ef[Transition.NOT_SET] = lambda atn, src, trg, arg1, arg2, arg3, sets, target : \
                NotSetTransition(target, sets[arg1])
            ef[Transition.WILDCARD] = lambda atn, src, trg, arg1, arg2, arg3, sets, target : \
                WildcardTransition(target)
            self.edgeFactories = ef

        if type> len(self.edgeFactories) or self.edgeFactories[type] is None:
            raise Exception("The specified transition type: " + str(type) + " is not valid.")
        else:
            return self.edgeFactories[type](atn, src, trg, arg1, arg2, arg3, sets, target)

    def stateFactory(self, type, ruleIndex):
        if self.stateFactories is None:
            sf = [None] * 13
            sf[ATNState.INVALID_TYPE] = lambda : None
            sf[ATNState.BASIC] = lambda : BasicState()
            sf[ATNState.RULE_START] = lambda : RuleStartState()
            sf[ATNState.BLOCK_START] = lambda : BasicBlockStartState()
            sf[ATNState.PLUS_BLOCK_START] = lambda : PlusBlockStartState()
            sf[ATNState.STAR_BLOCK_START] = lambda : StarBlockStartState()
            sf[ATNState.TOKEN_START] = lambda : TokensStartState()
            sf[ATNState.RULE_STOP] = lambda : RuleStopState()
            sf[ATNState.BLOCK_END] = lambda : BlockEndState()
            sf[ATNState.STAR_LOOP_BACK] = lambda : StarLoopbackState()
            sf[ATNState.STAR_LOOP_ENTRY] = lambda : StarLoopEntryState()
            sf[ATNState.PLUS_LOOP_BACK] = lambda : PlusLoopbackState()
            sf[ATNState.LOOP_END] = lambda : LoopEndState()
            self.stateFactories = sf

        if type> len(self.stateFactories) or self.stateFactories[type] is None:
            raise Exception("The specified state type " + str(type) + " is not valid.")
        else:
            s = self.stateFactories[type]()
            if s is not None:
                s.ruleIndex = ruleIndex
            return s

    def lexerActionFactory(self, type, data1, data2):
        if self.actionFactories is None:
            af = [ None ] * 8
            af[LexerActionType.CHANNEL] = lambda data1, data2: LexerChannelAction(data1)
            af[LexerActionType.CUSTOM] = lambda data1, data2: LexerCustomAction(data1, data2)
            af[LexerActionType.MODE] = lambda data1, data2: LexerModeAction(data1)
            af[LexerActionType.MORE] = lambda data1, data2: LexerMoreAction.INSTANCE
            af[LexerActionType.POP_MODE] = lambda data1, data2: LexerPopModeAction.INSTANCE
            af[LexerActionType.PUSH_MODE] = lambda data1, data2: LexerPushModeAction(data1)
            af[LexerActionType.SKIP] = lambda data1, data2: LexerSkipAction.INSTANCE
            af[LexerActionType.TYPE] = lambda data1, data2: LexerTypeAction(data1)
            self.actionFactories = af

        if type> len(self.actionFactories) or self.actionFactories[type] is None:
            raise Exception("The specified lexer action type " + str(type) + " is not valid.")
        else:
            return self.actionFactories[type](data1, data2)



#!/usr/bin/env python
# -coding:utf-8
import sys
from QBCodeSpecs.objcRecoginition.ObjectiveCParserListener import *
from QBCodeSpecs.driver.ViolationDataMananger import *
from QBCodeSpecs.driver.Violation import *
import re
import linecache


class MacroDefinitionSingletonRuleListener(ObjectiveCParserListener):

	def __init__(self, violation):
		super(MacroDefinitionSingletonRuleListener,self).__init__()
		self.violation = violation
		self.startline = 0
		self.startcolumn = 0


	def send_violation(self, startline, startcolumn):

		path = self.violation.path
		configInfo = self.violation.configinfo

		violInfo = Violation(path, configInfo)
		violInfo.startline = startline
		violInfo.startcolumn = startcolumn

		# violInfo.set_rulename("")
		violInfo.set_rulecategory("单例写法")
		# codeStr = linecache.getline(path,self.startline)
		# print "source code:" + codeStr
		violInfo.message = "请使用DEF_SINGLETON来定义单例"
		# "方法定义 " + codeStr + " 返回值的类型应为instancetype"
		#"Conforming to regular expressions:" + "\"" + self.regex + "\"" 

		democode = "@implementation MttPatchDataManager\n\
DEF_SINGLETON(MttPatchDataManager)\n\
...\n\
@end"
		violInfo.set_democode(democode)
		noncompliant = "+ (instancetype)sharedSettings\n\
{\n\
    static CBAppSettings *settings = nil;\n\
    static dispatch_once_t onceToken;\n\
    dispatch_once(&onceToken, ^{\n\
        settings = [[CBAppSettings alloc]init];\n\
    });\n\
    return settings;\n\
}"
		violInfo.set_noncompliant_code(noncompliant)
		violationDataMananger.addViolation(violInfo)

	def enterImplementationDefinitionList(self, ctx):


		classDefList = ctx.classMethodDefinition()
		for classDef in classDefList:
			# print "methodDefinition:" + classDef.methodDefinition().getText()
			methodSel = classDef.methodDefinition().methodSelector()

			selectorStr = methodSel.getText()
			# print methodSel.getText()
			if selectorStr.startswith("share"):
				token = methodSel.start
				startline = token.line
				startcolumn = token.column
				

				methodText = classDef.methodDefinition().getText()
				contentlist = re.findall(r"dispatch_once(.+?)\);", methodText)
				# print contentlist
				for content in contentlist:
					# print "content:" + content
					if len(content) > 0:
						if "alloc]init" in content or "new]" in content:
							self.send_violation(startline, startcolumn)
							break
				

				


